<?php

namespace Drupal\reroute_email_symfony_mailer\Plugin\RerouteEmailHandler;

use Drupal\reroute_email\RerouteEmailHandlerPluginBase;
use Drupal\symfony_mailer\Exception\SkipMailException;

/**
 * Defines the RerouteEmailHandler plug-in for RerouteEmailAdjuster mails.
 *
 * @RerouteEmailHandler(
 *   id = "reroute_email_reroute_email_adjuster",
 *   applied = {"symfony_mailer_adjuster"},
 * )
 */
class SymfonyMailerAdjuster extends RerouteEmailHandlerPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getMailKey() {
    return $this->configuration['email']->getType() . '_' . $this->configuration['email']->getSubType();
  }

  /**
   * {@inheritdoc}
   */
  public function getMailModule() {
    return $this->configuration['email']->getType();
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressTo() {
    return $this->getStringEmails($this->configuration['email']->getTo());
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressTo(string $to): void {
    $this->configuration['email']->setTo($to);
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressCc() {
    return $this->getStringEmails($this->configuration['email']->getCc());
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressCc(?string $cc): void {
    // There is no way to erase 'cc' value right now (if $cc is NULL).
    /* @see: https://www.drupal.org/project/symfony_mailer/issues/3285188#comment-15265746 */
    $this->configuration['email']->setCc($cc ?? '');
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressBcc() {
    return $this->getStringEmails($this->configuration['email']->getBcc());
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressBcc(?string $bcc): void {
    // There is no way to erase 'bcc' value right now (if $bcc is NULL).
    /* @see: https://www.drupal.org/project/symfony_mailer/issues/3285188#comment-15265746 */
    $this->configuration['email']->setBcc($bcc ?? '');
  }

  /**
   * {@inheritdoc}
   */
  public function getHeader(string $name) {
    if ($this->configuration['email']->getHeaders()->has($name)) {
      return $this->configuration['email']->getHeaders()->get($name)->getBodyAsString();
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function headerExist(string $name): bool {
    return $this->configuration['email']->getHeaders()->has($name);
  }

  /**
   * {@inheritdoc}
   */
  public function setHeader(string $name, string $value): void {
    $this->configuration['email']->getHeaders()->addTextHeader($name, $value);
  }

  /**
   * {@inheritdoc}
   */
  public function prependBody(string $body): void {
    if (!empty($this->configuration['email']->getTextBody())) {
      $this->configuration['email']->setTextBody($body . $this->configuration['email']->getTextBody());
    }
    else {
      $this->configuration['email']->setHtmlBody($body . $this->configuration['email']->getHtmlBody());
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\symfony_mailer\Exception\SkipMailException
   */
  public function setAsAborted(bool $skip = TRUE): void {
    throw new SkipMailException('Abort sending of the email if the no rerouting addresses provided.');
  }

  /**
   * {@inheritdoc}
   */
  public function getRawData(): array {
    return $this->configuration['email']->__serialize();
  }

  /**
   * Get string emails.
   *
   * @param array $email
   *   An array of email objects.
   *
   * @return string|null
   *   A string of emails or NULL.
   */
  protected function getStringEmails(array $email): ?string {
    $addresses = NULL;

    if (!empty($email)) {
      foreach ($email as $item) {
        $addresses[] = $item->getEmail();
      }

      $addresses = implode(', ', $addresses);
    }

    return $addresses;
  }

}
