<?php

namespace Drupal\reroute_email;

/**
 * Defines the interface for RerouteEmailHandler plugins.
 */
interface RerouteEmailHandlerPluginInterface {

  const HEADER_REASON = "X-Rerouted-Reason";
  const HEADER_STATUS = "X-Rerouted-Status";
  const HEADER_BODY_UPDATED = "X-Rerouted-Body-Details";
  const HEADER_MAIL_ID = "X-Rerouted-Mail-Key";
  const HEADER_FORCE_SKIP = "X-Rerouted-Force";
  const HEADER_BASE_URL = "X-Rerouted-Website";
  const HEADER_ORIGINAL_TO = "X-Rerouted-Original-to";
  const HEADER_ORIGINAL_CC = "X-Rerouted-Original-cc";
  const HEADER_ORIGINAL_BCC = "X-Rerouted-Original-bcc";

  /**
   * Get email ID.
   */
  public function getMailKey();

  /**
   * Get email module.
   */
  public function getMailModule();

  /**
   * Get email To address.
   */
  public function getAddressTo();

  /**
   * Get email To address.
   */
  public function setAddressTo(string $to): void;

  /**
   * Get email CC address(es).
   */
  public function getAddressCc();

  /**
   * Set email CC address(es).
   */
  public function setAddressCc(?string $cc): void;

  /**
   * Get email BCC address(es).
   */
  public function getAddressBcc();

  /**
   * Set email BCC address(es).
   */
  public function setAddressBcc(?string $bcc): void;

  /**
   * Get email headers.
   */
  public function getHeader(string $name);

  /**
   * Check email header availability.
   */
  public function headerExist(string $name): bool;

  /**
   * Set email headers.
   */
  public function setHeader(string $name, string $value): void;

  /**
   * Set email body.
   */
  public function prependBody(string $body): void;

  /**
   * Abort sending of the email.
   */
  public function setAsAborted(bool $skip = TRUE): void;

  /**
   * Get rerouted email raw data for logging purposes.
   */
  public function getRawData(): array;

}
